// Include the Arduino servo lib
#include <Servo.h>

// The left servo
Servo LeftTrack;
// The right servo
Servo RightTrack;

// The raw left value
float rawLeft;
// The raw right value
float rawRight;

// The max value of the input device
float MaxJoy = 1;
// The min value of the input device
float MinJoy = -1;

// The max vale of the range to be mapped to
float MaxValue = 180;
// The min vale of the range to be mapped to
float MinValue = 0;

// The raw left value
float RawLeft;
// The raw right value
float RawRight;

// The mapped range left value
float ValLeft;
// The mapped range right value
float ValRight;

// Properties used to invert the
// direction of the servos.
boolean invXL = false;
boolean invXR = false;

// Flag used to show the debug of the
// tank values.
boolean debugTank = true;

void tankDriveSetup()
{
   Serial.println("Tank Drive Script Ready!");
   
   // Set values to start stationary   
   ValLeft = 90; ValRight = 90;
   
   // Attach each servo to the required pin   
   LeftTrack.attach(2);
   RightTrack.attach(3);
   
   // invert the Left motor to cater for the
   // mounting of the servos. In my case the
   // left servo needs inverting.   
   invXL = true;
   
   // Apply the stationary values (90) so that
   // the robot is stationary on start.   
   LeftTrack.write(ValLeft);
   RightTrack.write(ValRight);
}

void tankDriveLoop()
{  
}

void CalculateTankJoyDrive(char *prop, char *prod)
{
  // Convert property to float
  float x = atof(prop);  
  // Convert property to float
  float y = atof(prod);
  
  // Use the raw joystick values to calculate the
  // track speeds and directions
  CalculateTankDrive(x, y);
  
   // Apply the values to the servos   
   LeftTrack.write(ValLeft);
   RightTrack.write(ValRight);
   
   // Show the debug values if flag set to true.
  if(debugTank)
  {
    Serial.print("JtoV: ");
    Serial.print(ValLeft);
    Serial.print(" : ");
    Serial.print(ValRight);
    Serial.println();
  }
}

void ApplyTankDriveMapValue(char *prop, char *prod)
{
  // Convert property to int
  int lDrive = atoi(prop);  
  // Convert property to int
  int rDrive = atoi(prod);
  
  // Set ValLeft to the received left value
  rawLeft = lDrive;
  // Set ValRight to the received right value
  rawRight = rDrive;
  
  // Cater for inverse of direction if needed
  if(invXL){ rawLeft = MaxValue - rawLeft; }
  if(invXR){ rawRight = MaxValue - rawRight; }
  
  // Set the mapped values to that of the
  // modified raw left and right values.
  ValLeft = rawLeft;
  ValRight = rawRight;
  
  // Apply the received left value
  LeftTrack.write(ValLeft);
  // Apply the received right value
  RightTrack.write(ValRight);
  
  // Show the debug values if flag set to true.
  if(debugTank)
  {
    Serial.print("Val: ");
    Serial.print(ValLeft);
    Serial.print(" : ");
    Serial.print(ValRight);
    Serial.println();
  }
}

void ApplyTankDriveRawValue(char *prop, char *prod)
{
}

void CalculateTankDrive(float x, float y)
 {
   // first Compute the angle in deg
   // First hypotenuse        
   float z = sqrt(x * x + y * y);
   
   // angle in radians   
   float rad = acos(abs(x) / z);
   
   // Cataer for NaN values   
   if(isnan(rad) == true){ rad=0; }
   
   // and in degrees   
   float angle = rad * 180 / PI;
   
   // Now angle indicates the measure of turn
   // Along a straight line, with an angle o, the turn co-efficient is same
   // this applies for angles between 0-90, with angle 0 the co-eff is -1
   // with angle 45, the co-efficient is 0 and with angle 90, it is 1
   
   float tcoeff = -1 + (angle / 90) * 2;
   float turn = tcoeff * abs(abs(y) - abs(x));
   turn = round(turn * 100) / 100;
   
   // And max of y or x is the movement   
   float mov = max(abs(y), abs(x));
   
   // First and third quadrant   
   if ((x >= 0 && y >= 0) || (x < 0 && y < 0))
   { rawLeft = mov; rawRight = turn; }
   else
   { rawRight = mov; rawLeft = turn; }
   
   // Reverse polarity   
   if (y < 0){ rawLeft = 0 - rawLeft; rawRight = 0 - rawRight; }
   
   // Update the values   
   RawLeft = rawLeft;
   RawRight = rawRight;
   
   // Map the values onto the defined rang   
   ValLeft = map(rawLeft, MinJoy, MaxJoy, MinValue, MaxValue);
   ValRight = map(rawRight, MinJoy, MaxJoy, MinValue, MaxValue);
   
   // Cater for inverse of direction if needed   
   if(invXL){ rawLeft *= -1; ValLeft = MaxValue - ValLeft; }
   if(invXR){ rawRight *= -1; ValRight = MaxValue - ValRight}
 }

